# -*- coding: utf-8 -*-

import json
import random

from ydl_ai_recommender.src.core.manager import (
    CounselorProfileManager,
    UserEventManager,
)
from ydl_ai_recommender.src.core.profile import encode_user_profile
from ydl_ai_recommender.src.core.counselor_profile import encode_counselor_profile


def make_data(start_date, end_date, type):
    
    # 平台用户数量太多，仅处理与本次任务相关的用户
    manager = UserEventManager()
    user_profiles = manager.load_xlsx_data(f'expose_click_user_{start_date}_{end_date}.xlsx')
    user_embeddings = {}
    for _, profile in user_profiles.iterrows():
        user_embeddings[str(profile['uid'])] = encode_user_profile(profile)

    counselor_profile_manager = CounselorProfileManager()
    counselor_profiles = counselor_profile_manager.load_xlsx_data('all_counselor_profile.xlsx')

    counselor_embeddings = {}
    for _, profile in counselor_profiles.iterrows():
        counselor_embeddings[str(profile['doctor_id'])] = encode_counselor_profile(profile)

    # 曝光点击数据
    train_data = []
    expost_click = manager.load_data(start_date=start_date, end_date=end_date)
    for _, profile in expost_click.iterrows():
        uid = str(profile['user_ids'])
        if ',' in uid:
            continue

        if uid not in user_embeddings:
            continue
        
        expose_ids = set(str(profile['ae_doctor_ids']).split(','))
        click_ids = set(str(profile['doctor_card_doctor_ids']).split(',') + str(profile['private_chat_doctor_ids']).split(','))
        click_ids2 = [_id for _id in click_ids if _id in counselor_embeddings]

        for _id in click_ids2:
            train_data.append([user_embeddings[uid] + counselor_embeddings[_id], 1])
        
        uninterested_ids = list(expose_ids - click_ids)
        for _id in random.choices(uninterested_ids, k=min(len(click_ids2), len(uninterested_ids))):
            if _id not in counselor_embeddings:
                continue
            train_data.append([user_embeddings[uid] + counselor_embeddings[_id], 0])

    print('data count: ', len(train_data))
    print('label=0\t{}'.format(len(list(filter(lambda x: x[1] == 0, train_data)))))
    print('label=1\t{}'.format(len(list(filter(lambda x: x[1] == 1, train_data)))))

    with open(f'data/all_{type}.json', 'w') as f:
        json.dump(train_data, f)
    

def make_train_data():
    start_date = '2023-01-01'
    end_date = '2023-02-01'
    make_data(start_date, end_date, 'train')


def make_test_data():
    start_date = '2023-02-01'
    end_date = '2023-02-10'
    make_data(start_date, end_date, 'test')


if __name__ == '__main__':
    make_train_data()
    make_test_data()